/* absyn.c - Abstract syntax of a gringo script
   Copyright (C) 1999 Gabriele Mambrini g.mambro@panservice.it

   This file is part of gringosh.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. */

#include <stdlib.h>
#include <assert.h>
#include "absyn.h"

/* button */
as_button as_mk_button(char *name, int x, int y, int w, char *text)
{
	as_button ris;
	ris = (as_button) malloc(sizeof(struct as_button_));
	assert(ris);
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->text = text;
	ris->name = name;
	return ris;
}


/* checkbox */
as_checkbox as_mk_checkbox(char *name, int x, int y, char *text, int value)
{
	as_checkbox ris;
	ris = (as_checkbox) malloc(sizeof(struct as_checkbox_));
	assert(ris);
	ris->x = x;
	ris->y = y;
	ris->value = value;
	ris->text = text;
	ris->name = name;
	return ris;
}

/* entry */
as_entry as_mk_entry(char *name, int x, int y, int w, char *text)
{
	as_entry ris;
	ris = (as_entry) malloc(sizeof(struct as_entry_));
	assert(ris);
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->text = text;
	ris->name = name;
	return ris;
}


/* label */
as_label as_mk_label(int x, int y, int w, char *text)
{
	as_label ris;
	ris = (as_label) malloc(sizeof(struct as_label_));
	assert(ris);
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->text = text;
	return ris;
}

/* window */
as_window as_mk_window(char *name, int x, int y, int w, int h, char *title)
{
	as_window ris;
	ris = (as_window) malloc(sizeof(struct as_window_));
	assert(ris);
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->h = h;
	ris->title = title;
	ris->name = name;
	return ris;
}


/* grid */
as_grid as_mk_grid(char *name, int x, int y, int w, int h, as_glist e)
{
	as_grid ris;
	ris = (as_grid) malloc(sizeof(struct as_grid_));
	assert(ris);
	ris->name = name;
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->h = h;
	ris->elems = e;
	return ris;
}


/* grid */
as_list as_mk_list(char *name, int x, int y, int w, int h, as_glist e)
{
	as_list ris;
	ris = (as_list) malloc(sizeof(struct as_list_));
	assert(ris);
	ris->name = name;
	ris->x = x;
	ris->y = y;
	ris->w = w;
	ris->h = h;
	ris->elems = e;
	return ris;
}

/* strlist */
as_glist as_mk_glist(void *v, as_glist next)
{
	as_glist ris;
	ris = (as_glist) malloc(sizeof(struct as_glist_));
	ris->val = v;
	ris->next = next;
	return ris;
}


as_command as_mk_command(int kind, void *command)
{
	as_command ris;
	ris = (as_command) malloc(sizeof(struct as_command_));
	assert(ris);
	ris->kind = kind;

	switch (kind) {
	case as_kbutton:
		ris->u.button = command;
		break;	
	case as_kentry:
		ris->u.entry = command;
		break;		
	case as_klabel:
		ris->u.label = command;
		break;		
	case as_kwindow:
		ris->u.window = command;
		break;		
	case as_kgrid:
		ris->u.grid = command;
		break;
	case as_klist:
		ris->u.list = command;
		break;
	case as_kcheckbox:
		ris->u.checkbox = command;
		break;
	}
	ris->next = NULL;
	return ris;
}
