/* absyn.h - Abstract syntax of a gringo script
   Copyright (C) 1999 Gabriele Mambrini g.mambro@panservice.it

   This file is part of gringosh.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. */

#ifndef _ABSYN_H_
#define _ABSYN_H_


/* button */
typedef struct as_button_ {
	int x, y, w;
	char *text, *name;
} *as_button;


/* entry */
typedef struct as_entry_ {
	int x, y, w;
	char *text, *name;
} *as_entry;

/* checkbox */
typedef struct as_checkbox_ {
	int x, y, value;
	char *text, *name;
} *as_checkbox;

/* label */
typedef struct as_label_ {
	int x, y, w;
	char *text;
} *as_label;

/* window */
typedef struct as_window_ {
	int x, y, h, w;
	char *name;
	char *title;
} *as_window;

/* glist: generic list */
typedef struct as_glist_ {
	void *val;
	struct as_glist_ *next;
} *as_glist;

/* grid */
typedef struct as_grid_ {
	int x, y, h, w;
	char *name;
	as_glist elems;
} *as_grid;

/* list */
typedef struct as_list_ {
	int x, y, h, w;
	char *name;
	as_glist elems;
} *as_list;

/* A script is a list of commands... */
typedef struct as_command_ {
	enum {
		as_kbutton, as_kentry, as_klabel,
		as_kwindow, as_kgrid, as_klist, as_kcheckbox
	} kind;
	union {
		as_button button;
		as_entry entry;
		as_label label;
		as_window window;
		as_grid grid;
		as_list list;
		as_checkbox checkbox;
	} u;
	struct as_command_ *next;
} *as_command;


/* Constructors */
as_command as_mk_command(int kind, void *command);

as_button as_mk_button(char *name, int x, int y, int w, char *text);
as_checkbox as_mk_checkbox(char *name, int x, int y, char *text, int value);
as_entry as_mk_entry(char *name, int x, int y, int w, char *text);
as_label as_mk_label(int x, int y, int w, char *text);
as_window as_mk_window(char *name, int x, int y, int w, int h, char *title);
as_grid as_mk_grid(char *name, int x, int y, int w, int h, as_glist e);
as_list as_mk_list(char *name, int x, int y, int w, int h, as_glist e);

as_glist as_mk_glist(void *v, as_glist next);
#endif
