%{

#include <string.h>
#include "absyn.h"
#include "gringosh-tab.h"	
#include "token_pos.h"

#define MAXSTRLEN 1024

char str_buffer[MAXSTRLEN],*str_ptr;

void str_append(char c) {
	if(str_ptr-str_buffer>=0)
		*str_ptr++=c;
	else {
		tp_printerror(tokenpos,"String too long.");
		exit(1); 
	}
}

%}

%option noyywrap
%x S S2

 /* S is quoted by " while S1 is quoted by ' */

id	[a-zA-Z_]([_a-zA-Z0-9])*

%%

\"		{	
			tp_setpos();
			str_ptr = str_buffer;
			BEGIN(S); 			
		}

'		{	
			tp_setpos();
			str_ptr = str_buffer;
			BEGIN(S2);
		}


<S>\"		{
			tp_setpos();
			BEGIN(INITIAL);
                        *str_ptr='\0';			
                        yylval.string_val = strdup(str_buffer);	
                        return STRING;
		}

<S2>'		{
			tp_setpos();
			BEGIN(INITIAL);
                        *str_ptr='\0';			
                        yylval.string_val = strdup(str_buffer);	
                        return STRING;
		}

<S,S2>\n        {       
                        tp_printerror(tokenpos,"Untermined string.");
			exit(1);
                }

<S,S2>\\\n      {	
			tp_setpos(); str_append('\n'); continue;
		}

<S>\\\"      	{	
			tp_setpos(); str_append('"'); continue;
		}

<S2>\\'		{	
			tp_setpos(); str_append('\''); continue;
		}

<S>[^\\\n\"]+   {	
			char *p = yytext;
			tp_setpos();
                        while(*p) str_append(*p++);
		}


<S2>[^\\\n']+   {	
			char *p = yytext;
			tp_setpos();
                        while(*p) str_append(*p++);
		}


#.*\n		{tp_newline(); continue;}

"button"	{tp_setpos(); return BUTTON;}
"entry"		{tp_setpos(); return ENTRY;}
"label"		{tp_setpos(); return LABEL;}
"window"	{tp_setpos(); return WINDOW;}
"grid"		{tp_setpos(); return GRID;}
"list"		{tp_setpos(); return LIST;}
"checkbox"	{tp_setpos(); return CHECKBOX;}
"end"		{tp_setpos(); return END;}

[0-9]+		{
			tp_setpos(); 
			yylval.int_val = atoi(yytext); 
			return INT;
		}

{id}		{		tp_setpos(); 
				yylval.string_val = strdup(yytext); 
				return ID;
		}


\n		{tp_newline(); return NL;}

(" "|\t)*       {tp_setpos();  continue;}

.		{	
		      tp_printerror(tokenpos,"Lexical error.");
			      tp_setpos();
		}

%%
