/* main.c
   Copyright (C) 1999 Gabriele Mambrini gm@badpenguin.org

   This file is part of gringosh.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>

#include <gringo.h>

#include "absyn.h"
#include "token_pos.h"
#include "gringosh-tab.h"
#include "wm.h"

#define HELP "\
`gringosh' a dialog replacement\n\
Copyright (C) 2000 Gabriele Mambrini\n\
usage: gringosh [option] file [file2]\n\
 -h,--help	give this help\n\
 -v,--version	display version number\n\
 file		file to read. If file is `-' use standard input.\n\
 file2		where to dump gringosh status. Default is stderr."
	

extern void yyparse(void);
void read_param(int argc, char **argv);

int main(int argc, char **argv) {
	extern as_command absyn_root;
	read_param(argc, argv);
	gringo_Init(&argc, &argv);
	yyparse();
	tp_closefile();
	wm_run(absyn_root);
	gringo_Loop();
	gringo_Quit();
	return 0;
}


void read_param(int argc, char **argv) {
	int c, index, n_args;
	static struct option long_options[] = {
		{"help",    0, 0, 'h'},
		{"version", 0, 0, 'v'},
		{"force-ncurses",    0, 0, 'n'},
		{"force-gtk", 0, 0, 'g'},
		{NULL, 0, 0, 0}
	};

	while (1) {
		c = getopt_long_only(argc, argv, "hvng", long_options, &index);
		if (c == EOF)
			break;
		switch (c) {
		
		/* Skip those two options that are for libgringo */
		case 'n': break;
		case 'g': break;
		
		case 'h':
			printf("%s %s %s\n", PACKAGE, VERSION, HELP);
			exit(0);
			break;
		case 'v':
			printf("%s %s\n", PACKAGE, VERSION);
			exit(0);
			break;
		default:
			fprintf(stderr,
				"%s: Use %s --help for more information.\n", 
				PACKAGE,*argv);
			exit(1);
		}
	}
	
	n_args = argc - optind;
	/* input file */
	if(n_args<1 || n_args>2) {
		fprintf(stderr, "gringosh: Usage: %s [filename]\n", argv[0]);
		exit(1);
	}
	if(!strcmp("-",argv[optind]))
		tp_openfile(NULL);
	else
		tp_openfile(argv[optind]);
	/* where to dump */
	if(n_args==2) {
		FILE *dumpfile;	
		dumpfile = fopen(argv[optind+1],"w");
		if(!dumpfile) {
			perror(PACKAGE);
			exit(1);
		}
		wm_dump_file = dumpfile;
	}	
}	
