/* gringo grammar */
%{
#include <string.h>
#include "absyn.h"
#include "token_pos.h"

int yylex(void);

void yyerror(char *s) {
	tp_printerror(tokenpos,"%s",s);
}

as_command absyn_root;

%}


/* Returned data */
%union {
	int int_val;
	char *string_val;
	as_command command;
	as_window window;
	as_entry entry;
	as_checkbox checkbox;
	as_label label;
	as_button button;
	as_grid grid;
	as_list list;
	as_glist glist;
}

/* tokens */

%token BUTTON ENTRY LABEL WINDOW GRID LIST CHECKBOX
%token END NL

%token <int_val> INT
%token <string_val> STRING ID

/* non terminals */
%type	<command>	command lines
%type	<window>	window
%type	<button>	button
%type	<checkbox>	checkbox
%type	<entry>		entry
%type	<label>		label
%type 	<grid>		grid
%type 	<list>		list
%type	<glist>		gridelems gridrow listelems

%start file

%%

file:	lines			{ absyn_root = $1;}
	;

lines:    command NL lines	{ $1->next=$3;$$=$1; }
	| NL lines		{ $$=$2; }	
	| /* */			{ $$=NULL;}
	;


command:  window 	{ $$=as_mk_command(as_kwindow,$1);}
	| entry 	{ $$=as_mk_command(as_kentry,$1);}
	| button 	{ $$=as_mk_command(as_kbutton,$1);}
	| checkbox 	{ $$=as_mk_command(as_kcheckbox,$1);}
	| label 	{ $$=as_mk_command(as_klabel,$1);}
	| grid 		{ $$=as_mk_command(as_kgrid,$1);}
	| list 		{ $$=as_mk_command(as_klist,$1);}	
	;


sep:	 /*, may be omitted */
	| ',' 
	;

window: WINDOW ID sep INT sep INT sep INT sep INT sep STRING 
		{$$=as_mk_window($2,$4,$6,$8,$10,$12);}
	;

/* entry name,x,y[,text][,width] */
entry:	  ENTRY ID sep INT sep INT sep STRING
		{ $$=as_mk_entry($2,$4,$6,strlen($8),$8); }
	| ENTRY ID sep INT sep INT sep INT
		{ $$=as_mk_entry($2,$4,$6,$8,NULL); }
	| ENTRY ID sep INT sep INT sep STRING sep INT
		{ $$=as_mk_entry($2,$4,$6,$10,$8); }
	;

/* button name,x,y,name[,width] */
button:	  BUTTON ID sep INT sep INT sep STRING
		{ $$=as_mk_button($2,$4,$6,0,$8); }
	| BUTTON ID sep INT sep INT sep STRING sep INT
		{ $$=as_mk_button($2,$4,$6,$10,$8); }
	;

/* checkbox name,x,y,name,value */
checkbox: CHECKBOX ID sep INT sep INT sep STRING sep INT
		{ $$=as_mk_checkbox($2,$4,$6,$8,$10); }
	;


/* label name,x,y[,width] */
label:	  LABEL INT sep INT sep STRING 
		{$$=as_mk_label($2,$4,0,$6);}
	| LABEL INT sep INT sep STRING sep INT
		{$$=as_mk_label($2,$4,$8,$6);}	
	;

grid:	GRID ID sep INT sep INT sep INT sep INT NL gridelems END GRID
		{$$=as_mk_grid($2,$4,$6,$8,$10,$12);}
	;

gridelems:  
	  /* empty */	{$$=NULL;}
	|  gridrow gridelems
		{$$=as_mk_glist($1,$2);}
	;

gridrow:  NL 			{ $$=NULL;}
	| STRING NL		{ $$=as_mk_glist($1,NULL);}
	| STRING sep gridrow 	{ $$=as_mk_glist($1,$3);}
	;

list:	LIST ID sep INT sep INT sep INT sep INT NL listelems END LIST
		{$$=as_mk_list($2,$4,$6,$8,$10,$12);}
	;

listelems:
	  /*empty*/ {$$=NULL}
	| STRING NL listelems
		{$$=as_mk_glist($1,$3);}
	;
%%
