/* token_pos.c
   Copyright (C) 1999 Gabriele Mambrini gm@badpenguin.org

   This file is part of gringosh.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "token_pos.h"
#include "config.h"

int tokenpos = 0, charpos = 0;
char *filename = NULL;

extern int yyleng;
extern FILE *yyin;
extern char *yytext;

/***************************************************/
/*      Private support functions and vars         */
/***************************************************/

/* ll  = lines list: a list of int where we put 
   tokenpos when we find newlines */

typedef struct __ll_element {
	int val;
	struct __ll_element *next;
} _ll_element;

_ll_element *_ll_head = NULL;
_ll_element *_ll_tail = NULL;

void _ll_add(int i)
{
	_ll_element *n;
	n = (_ll_element *) malloc(sizeof(_ll_element));
	n->val = i;
	n->next = NULL;

	if (_ll_tail)
		_ll_tail->next = n;
	_ll_tail = n;


	if (!_ll_head)
		_ll_head = n;

}


/**********************************************/
/*         tp_print_error                     */
/**********************************************/

/* Standard file-related errors printing routine */

void tp_printerror(int pos, char *message, ...)
{
	va_list ap;

	int col, line = 1, lastpos = 0;
	_ll_element *s = _ll_head;
	while (s && s->val <= pos) {
		line++;
		lastpos = s->val;
		s = s->next;
	}

	col = pos - lastpos + 1;

	if (filename)
		fprintf(stderr, "%s: ", filename);

	fprintf(stderr, "%i.%i: ", line, col);

	va_start(ap, message);
	vfprintf(stderr, message, ap);
	va_end(ap);

	fprintf(stderr, "\n");
}

/*************************************************/
/*           Lexer specific functions            */
/*************************************************/

void tp_newline(void)
{
	tokenpos = charpos;

	charpos += yyleng;
	_ll_add(charpos);
}


void tp_setpos(void)
{
	tokenpos = charpos;
	charpos += yyleng;
}


void tp_openfile(char *fname)
{
	if (fname) {
		yyin = fopen(fname, "r");
		filename = fname;
		if (!yyin) {
			perror(PACKAGE);
			exit(1);
		}
	} else
		yyin = stdin;
}

void tp_closefile(void)
{
	if (filename)
		fclose(yyin);
}
