/* wm.c - widget maker
   Copyright (C) 1999 Gabriele Mambrini gm at badpenguin.org

   This file is part of gringosh.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <gringo.h>
#include <badpenguin-chain.h>

#include "wm.h"
#include "config.h"
#define CHECK_WINDOW if (!cur_window) {\
        fprintf(stderr, "You must create a window!\n");\
	exit(1);\
}
#define SETNAME(obj,objname) TOBJECT(obj)->name=objname;

static TWindow *cur_window = NULL;
static void *newobj;

FILE	*wm_dump_file = NULL;

static void dump_window(TWindow * w)
{
	TChain *c;
	TChainItem *i;
	TObject *obj;

	fprintf(wm_dump_file, "#in window %s:\n", TOBJECT(w)->name);
	c = w->widgets;
	i=c->head;
	for (i=c->head; i; i = i->next) {
		obj = TOBJECT(i->data);
		switch (obj->type) {
		case TYPE_BUTTON:
			fprintf(wm_dump_file, "%s=%i\n", obj->name,
				button_GetValue(TBUTTON(obj)));
			break;
		case TYPE_EDITBOX:
			fprintf(wm_dump_file, "%s=\"%s\"\n", obj->name,
				edit_GetValue(TEDIT(obj)));
			break;
		case TYPE_CHECKBOX:
			fprintf(wm_dump_file, "%s=%i\n", obj->name,
				checkbox_GetValue(TCHECKBOX(obj)));
			break;
		case TYPE_LISTBOX:
			fprintf(wm_dump_file,"%s=\"%s\"\n",obj->name,
			   listbox_GetValue(TLISTBOX(obj)));
			break;
		case TYPE_GRID:
			/* TODO: trovare riga attiva */
			fprintf(wm_dump_file, "#grid unsopported yet.\n");
			/*fprintf(wm_dump_file,"%s=%s\n",obj->name, */
			break;
		}
	}
}

static void terminate(void)
{
	TChain *c;
	TChainItem *i;
	int counter,max;

	if(!wm_dump_file)
		wm_dump_file=stderr;

#warning problema TChain: leggere commento
/* TODO: questo sistema di leggere la lista con un contantore
 * invece che su un test sul puntatore c' perch l'ultimo
 * elemento non punta a NULL. Perch */
	c = TOBJECT(cur_window)->me->chain;
	max = c->count;
	i= c->head;
	for (counter=0;counter<max;counter++) {
		dump_window(TWINDOW(i->data));
		if(counter<max-1) i=i->next;
	}

	fprintf(wm_dump_file, "\nGRINGOSH=1\n");

	gringo_Terminate(0);
	gringo_Quit();
}


/**************************************************
                widget makers                         
 **************************************************/

/* TODO: change gringo API ad use a decent AddText :-) */

#define MAX_COLS 15

static void make_grid(as_grid g)
{
	as_glist row, e;
	char *args[MAX_COLS];
	int col_counter,n_cols;
	
	newobj = grid_Create(cur_window, g->x, g->y, g->w, g->h);
	SETNAME(newobj, g->name);

	row = g->elems;
	if (!row) return;

	n_cols=0;
	for (e = row->val; e; e = e->next) {
		if(++n_cols>MAX_COLS)
			fprintf(stderr,"%s : Too many columns in grid %s.",
					PACKAGE,g->name);		
		grid_AddColumn(newobj,e->val,strlen(e->val));
	}

	row = row->next;
	for (; row; row = row->next) {
		col_counter = 0;
		for (e = row->val; e; e = e->next)
			args[col_counter++] = e->val;
		if(col_counter>=n_cols)
			fprintf(stderr,"%s : Too many columns in grid %s.",
					PACKAGE,g->name);	
		/* horrible things*/
		grid_AddText(newobj,args[0],args[1],args[2],args[3],args[4],
			args[5],args[6],args[7],args[8],args[9],
			args[10],args[11],args[12],args[13],args[14]);

	}

}

static void make_list(as_list l)
{
	as_glist e;
	newobj = listbox_Create(cur_window, l->x, l->y, l->w, l->h);
	SETNAME(newobj, l->name);
	for (e = l->elems; e; e = e->next)
		listbox_AddItem(TLISTBOX(newobj), (char *) (e->val));

}

/**************************************************
                  wm_run                         
 **************************************************/

void wm_run(as_command c)
{
	for (; c; c = c->next) {
		switch (c->kind) {
		case as_kentry:
			CHECK_WINDOW;
			newobj = edit_Create(cur_window,
					     c->u.entry->x, c->u.entry->y,
					     c->u.entry->w, c->u.entry->text);
			SETNAME(newobj, c->u.entry->name);
			break;
		case as_kbutton:
			CHECK_WINDOW;
			newobj = button_Create(cur_window,
					       c->u.button->x, c->u.button->y,
					       c->u.button->w,
					       c->u.button->text);

			button_OnClick(TBUTTON(newobj), terminate);
			SETNAME(newobj, c->u.button->name);
			break;
		case as_kcheckbox:
			CHECK_WINDOW;
			newobj = checkbox_Create(cur_window,
						 c->u.checkbox->x,
						 c->u.checkbox->y,
						 c->u.checkbox->text,
						 c->u.checkbox->value);

			SETNAME(newobj, c->u.checkbox->name);
			break;

		case as_klabel:
			CHECK_WINDOW;
			newobj = label_Create(cur_window,
					      c->u.label->x, c->u.label->y,
					      c->u.label->w, c->u.label->text);
			break;
		case as_kwindow:
			cur_window =
			    window_Create(c->u.window->x, c->u.window->y,
					  c->u.window->w, c->u.window->h,
					  c->u.window->title);
			SETNAME(cur_window, c->u.window->name);
			break;
		case as_kgrid:
			CHECK_WINDOW;
			make_grid(c->u.grid);
			break;
		case as_klist:
			CHECK_WINDOW;
			make_list(c->u.list);
			break;
		default:
			printf("not implemented yet\n");
		}

	}
}
